@@ -63,7 +63,7 @@ def control(command, arg=None, ignore_error=False):
         exit_(str(e), notify_startup=True)
     else:
         if response is not None:
-            print_(response, end="")
+            print_(response, end="", flush=True)
         exit_(notify_startup=True)
 
 
@@ -87,6 +87,7 @@ def process_arguments(argv):
 
     options.add("print-playing", help=_("Print the playing song and exit"))
     options.add("start-playing", help=_("Begin playing immediately"))
+    options.add("start-hidden", help=_("Don't show any windows on start"))
 
     for opt, help in [
         ("next", _("Jump to next song")),
@@ -246,6 +247,8 @@ def process_arguments(argv):
             queue(command)
         elif command == "start-playing":
             actions.append(command)
+        elif command == "start-hidden":
+            actions.append(command)
         elif command == "no-plugins":
             actions.append(command)
         elif command == "run":
