@@ -41,7 +41,7 @@ INITIAL = {
         "pre_amp_gain": "0.0",
         "backend": "gstbe",
         "gst_pipeline": "",
-        "gst_buffer": "1.5", # stream buffer duration in seconds
+        "gst_buffer": "3", # stream buffer duration in seconds
         "gst_device": "",
         "gst_disable_gapless": "false",
     },
@@ -58,6 +58,7 @@ INITIAL = {
         "queue": "false", # on or off
         "queue_expanded": "false", # on or off
         "shufflequeue": "false", # on or off
+        "queue_stop_once_empty": "false", # on or off
         "sortby": "0album", # <reversed?>tagname, song list sort
 
         # Repeat on or off
@@ -80,6 +81,7 @@ INITIAL = {
             "~people\t<~year|[b][i]<~year>[/i][/b] - ><album>",
         "pane_selection": "", # selected pane values
         "pane_wide_mode": "0", # browser orientation
+        "equal_pane_width": "true", # equal pane width in paned browser
         "background": "", # "global" filter for SearchBar
         "albums": "", # album list
         "album_sort": "0", # album sorting mode, default is title
@@ -142,6 +144,12 @@ INITIAL = {
 
         # osx implementation might be buggy so let users disable it
         "disable_mmkeys": "false",
+
+        # the UI language to use, empty means system default
+        "language": "",
+
+        # the pattern for the main window title
+        "window_title_pattern": "~title~version~~people",
     },
     "rename": {
         "spaces": "false",
@@ -161,8 +169,12 @@ INITIAL = {
         "default_max_plugin_invocations": 30,
     },
     "editing": {
-        "split_on": "/ & ,", # words to split on
-        "id3encoding": "", # ID3 encodings to try
+        # characters to split tags on
+        "split_on": "/ & ,",
+
+        # ID3 encodings to try
+        "id3encoding": "",
+
         "human_title_case": "true",
         "save_to_songs": "true",
         "save_email": const.EMAIL,
