@@ -56,16 +56,16 @@ class MinVersions(object):
     MUTAGEN = Version("Mutagen", 1, 32,
         message="Use the Quod Libet unstable PPAs/repos to get a newer "
                 "mutagen version.")
-    GTK = Version("GTK+", 3, 10)
-    PYGOBJECT = Version("PyGObject", 3, 12)
-    GSTREAMER = Version("GStreamer", 1, 0)
+    GTK = Version("GTK+", 3, 14)
+    PYGOBJECT = Version("PyGObject", 3, 14)
+    GSTREAMER = Version("GStreamer", 1, 4)
 
 
-VERSION_TUPLE = Version("", 3, 8, 1)
+VERSION_TUPLE = Version("", 3, 9, 0)
 VERSION = str(VERSION_TUPLE)
 
 # entry point for the user guide / wiki
-BRANCH_NAME = "quodlibet-3.8"
+BRANCH_NAME = "quodlibet-3.9"
 DOCS_BASE_URL = "https://quodlibet.readthedocs.org/en/%s"
 DOCS_LATEST = DOCS_BASE_URL % "latest"
 DOCS_BASE_URL %= BRANCH_NAME if BRANCH_NAME != "master" else "latest"
@@ -79,17 +79,9 @@ EMAIL = os.environ.get("EMAIL", "quodlibet@lists.sacredchao.net")
 # Displayed as registered / help email address
 SUPPORT_EMAIL = "quod-libet-development@googlegroups.com"
 
-MAIN_AUTHORS = u"""\
-Joe Wreschnig
-Michael Urman
-Iñigo Serna
-Steven Robertson
-Christoph Reiter
-Nick Boultbee""".split("\n")
-
 # about dialog, --version etc.
 WEBSITE = "https://quodlibet.readthedocs.org/"
-COPYRIGHT = u"""Copyright © 2004-2016 %s...""" % u", ".join(MAIN_AUTHORS)
+COPYRIGHT = u"Copyright 2004-2017"
 
 AUTHORS = sorted(u"""\
 Alexandre Passos
@@ -114,6 +106,7 @@ Corentin Néau
 David Kågedal
 David Schneider
 Decklin Foster
+Didier Villevalois
 Eduardo Gonzalez
 Eric Casteleijn
 Erich Schubert
@@ -121,6 +114,7 @@ Eric Le Lay
 Federico Pelloni
 Felix Krull
 Florian Demmer
+Fredrik Strupe
 Guillaume Chazarain
 Hans Scholze
 Iñigo Serna
@@ -167,15 +161,23 @@ Tomasz Miasko
 Tomasz Torcz
 Tshepang Lekhonkhobe
 Türerkan İnce
+Uriel Zajaczkovski
 Vasiliy Faronov
 Victoria Hayes
 Zack Weinberg
+Vimalan Reddy
+Jason Heard
+David Pérez Carmona
+Jakub Wilk
+IBBoard@github
+CreamyCookie@github
 """.strip().split("\n"))
 
 TRANSLATORS = sorted(u"""
 Åka Sikrom (nb)
 Alexandre Passos (pt)
 Andreas Bertheussen (nb)
+Olivier Humbert (fr)
 Anton Shestakov (ru)
 Bastian Kleineidam (de)
 Bastien Gorissen (fr)
@@ -226,9 +228,10 @@ Yasushi Iwata (ja)
 Николай Прокошенко (ru)
 Ростислав "zbrox" Райков (bg)
 Сергей Федосеев (ru)
-scootergrisen (da)
+scootergrisen@github (da)
 Marek Suchánek (cs)
 Till Berger (de)
+Jean-Michel Pouré (fr)
 """.strip().splitlines())
 
 ARTISTS = sorted(u"""\
