@@ -104,13 +104,15 @@ class _UdevListIterator(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         next_ = self.__current.get_next()
         if next_:
             self.__current = next_
             return next_
         raise StopIteration
 
+    next = __next__
+
 
 class UdevListEntry(Structure):
     pass
