@@ -5,12 +5,11 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from gi.repository import Gtk
-
 from quodlibet import _
 from quodlibet.qltk import Icons
 from quodlibet.util import tag, escape
 from quodlibet.qltk.songlist import get_columns
+from quodlibet.qltk.chooser import choose_target_file
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.compat import text_type
 
@@ -73,19 +72,8 @@ class ExportToHTML(SongsMenuPlugin):
         if not songs:
             return
 
-        chooser = Gtk.FileChooserDialog(
-            title="Export to HTML",
-            action=Gtk.FileChooserAction.SAVE)
-        chooser.add_button(_("_Cancel"), Gtk.ResponseType.REJECT)
-        chooser.add_button(_("_OK"), Gtk.ResponseType.ACCEPT)
-        chooser.set_default_response(Gtk.ResponseType.ACCEPT)
-        resp = chooser.run()
-        if resp != Gtk.ResponseType.ACCEPT:
-            chooser.destroy()
-            return
-
-        fn = chooser.get_filename()
-        chooser.destroy()
-
-        with open(fn, "wb") as f:
-            f.write(to_html(songs).encode("utf-8"))
+        target = choose_target_file(
+            self.plugin_window, _("Export to HTML"), _("_Save"))
+        if target is not None:
+            with open(target, "wb") as f:
+                f.write(to_html(songs).encode("utf-8"))
