@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
-#                2016 Nick Boultbee
+#           2016-2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -14,16 +14,27 @@ from quodlibet import qltk
 from quodlibet import util
 from quodlibet import _
 
-from quodlibet.plugins import PluginManager
+from quodlibet.plugins import PluginManager, plugin_enabled
+from quodlibet.plugins.cover import CoverSourcePlugin
+from quodlibet.plugins.editing import EditTagsPlugin
+from quodlibet.plugins.events import EventPlugin
+from quodlibet.plugins.gstelement import GStreamerPlugin
+from quodlibet.plugins.playlist import PlaylistPlugin
+from quodlibet.plugins.playorder import PlayOrderPlugin
+from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.qltk.views import HintedTreeView
 from quodlibet.qltk.window import UniqueWindow, PersistentWindowMixin
-from quodlibet.qltk.entry import ClearEntry
 from quodlibet.qltk.x import Align, Paned, Button, ScrolledWindow
 from quodlibet.qltk.models import ObjectStore, ObjectModelFilter
-from quodlibet.qltk import Icons
+from quodlibet.qltk.entry import UndoEntry
+from quodlibet.qltk import Icons, is_accel, show_uri
 from quodlibet.util import connect_obj
 
 
+class UndoSearchEntry(Gtk.SearchEntry, UndoEntry):
+    pass
+
+
 class PluginErrorWindow(UniqueWindow):
     def __init__(self, parent, failures):
         if self.is_not_unique():
@@ -80,22 +91,23 @@ class PluginErrorWindow(UniqueWindow):
         self.get_child().show_all()
 
 
-class ComboType(object):
+class EnabledType(object):
     TAG, ALL, NO, DIS, EN, SEP = range(6)
 
 
-class PluginFilterCombo(Gtk.ComboBox):
+class PluginEnabledFilterCombo(Gtk.ComboBox):
 
     def __init__(self):
         combo_store = Gtk.ListStore(str, int)
-        super(PluginFilterCombo, self).__init__(model=combo_store)
+        super(PluginEnabledFilterCombo, self).__init__(model=combo_store)
 
         cell = Gtk.CellRendererText()
+        cell.props.ellipsize = Pango.EllipsizeMode.END
         self.pack_start(cell, True)
         self.add_attribute(cell, "text", 0)
 
         def combo_sep(model, iter_, data):
-            return model[iter_][1] == ComboType.SEP
+            return model[iter_][1] == EnabledType.SEP
 
         self.set_row_separator_func(combo_sep, None)
 
@@ -107,25 +119,69 @@ class PluginFilterCombo(Gtk.ComboBox):
         active = max(self.get_active(), 0)
         combo_store = self.get_model()
         combo_store.clear()
-        combo_store.append([_("All"), ComboType.ALL])
-        combo_store.append(["", ComboType.SEP])
-        combo_store.append([_("Enabled"), ComboType.EN])
-        combo_store.append([_("Disabled"), ComboType.DIS])
+        combo_store.append([_("All"), EnabledType.ALL])
+        combo_store.append(["", EnabledType.SEP])
+        combo_store.append([_("Enabled"), EnabledType.EN])
+        combo_store.append([_("Disabled"), EnabledType.DIS])
         if tags:
-            combo_store.append(["", ComboType.SEP])
+            combo_store.append(["", EnabledType.SEP])
             for tag in sorted(tags):
-                combo_store.append([tag, ComboType.TAG])
+                combo_store.append([tag, EnabledType.TAG])
             if no_tags:
-                combo_store.append([_("No category"), ComboType.NO])
+                combo_store.append([_("No category"), EnabledType.NO])
         self.set_active(active)
 
-    def get_active_tag(self):
+    def get_active_row(self):
         iter_ = self.get_active_iter()
         if iter_:
             model = self.get_model()
             return list(model[iter_])
 
 
+class PluginTypeFilterCombo(Gtk.ComboBox):
+
+    def __init__(self):
+        combo_store = Gtk.ListStore(str, object)
+        super(PluginTypeFilterCombo, self).__init__(model=combo_store)
+
+        cell = Gtk.CellRendererText()
+        cell.props.ellipsize = Pango.EllipsizeMode.END
+        self.pack_start(cell, True)
+        self.add_attribute(cell, "text", 0)
+
+        def combo_sep(model, iter_, data):
+            return model[iter_][1] is None
+
+        self.set_row_separator_func(combo_sep, None)
+        self.__refill()
+
+    def __refill(self):
+        """Fill with plugin types"""
+
+        active = max(self.get_active(), 0)
+        combo_store = self.get_model()
+        combo_store.clear()
+        combo_store.append([_("All"), object])
+        combo_store.append(["", None])
+        for name, cls in sorted([
+                [_("Songs"), SongsMenuPlugin],
+                [_("Playlists"), PlaylistPlugin],
+                [_("Events"), EventPlugin],
+                [_("Play Order"), PlayOrderPlugin],
+                [_("Editing"), EditTagsPlugin],
+                [_("Effects"), GStreamerPlugin],
+                [_("Covers"), CoverSourcePlugin]]):
+            combo_store.append([name, cls])
+
+        self.set_active(active)
+
+    def get_active_type(self):
+        iter_ = self.get_active_iter()
+        if iter_:
+            model = self.get_model()
+            return model[iter_][1]
+
+
 class PluginListView(HintedTreeView):
 
     __gsignals__ = {
@@ -139,6 +195,7 @@ class PluginListView(HintedTreeView):
         self.set_headers_visible(False)
 
         render = Gtk.CellRendererToggle()
+        render.set_padding(1, 1)
 
         def cell_data(col, render, model, iter_, data):
             plugin = model.get_value(iter_)
@@ -154,20 +211,23 @@ class PluginListView(HintedTreeView):
         self.append_column(column)
 
         render = Gtk.CellRendererPixbuf()
+        render.set_padding(1, 1)
 
         def cell_data2(col, render, model, iter_, data):
             plugin = model.get_value(iter_)
             icon = plugin.icon or Icons.SYSTEM_RUN
             render.set_property('icon-name', icon)
+            render.set_property('stock-size', Gtk.IconSize.LARGE_TOOLBAR)
 
         column = Gtk.TreeViewColumn("image", render)
+        column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         column.set_cell_data_func(render, cell_data2)
         self.append_column(column)
 
         render = Gtk.CellRendererText()
         render.set_property('ellipsize', Pango.EllipsizeMode.END)
         render.set_property('xalign', 0.0)
-        render.set_padding(3, 3)
+        render.set_padding(2, 2)
         column = Gtk.TreeViewColumn("name", render)
 
         def cell_data3(col, render, model, iter_, data):
@@ -178,11 +238,26 @@ class PluginListView(HintedTreeView):
         column.set_expand(True)
         self.append_column(column)
 
+    def do_key_press_event(self, event):
+        if is_accel(event, "space", "KP_Space"):
+            selection = self.get_selection()
+            fmodel, fiter = selection.get_selected()
+            plugin = fmodel.get_value(fiter)
+            if plugin.can_enable:
+                self._emit_toggled(fmodel.get_path(fiter),
+                                   not plugin_enabled(plugin))
+            self.get_model().iter_changed(fiter)
+        else:
+            Gtk.TreeView.do_key_press_event(self, event)
+
     def __toggled(self, render, path):
         render.set_active(not render.get_active())
+        self._emit_toggled(path, render.get_active())
+
+    def _emit_toggled(self, path, value):
         model = self.get_model()
         iter_ = model.get_iter(path)
-        self.emit("plugin-toggled", model, iter_, render.get_active())
+        self.emit("plugin-toggled", model, iter_, value)
 
     def select_by_plugin_id(self, plugin_id):
 
@@ -243,8 +318,9 @@ class PluginPreferencesContainer(Gtk.VBox):
             text = "<big><b>%s</b></big>" % name
             if plugin.description:
                 text += "<span font='4'>\n\n</span>"
-                text += util.escape(plugin.description)
+                text += plugin.description
             label.set_markup(text)
+            label.connect("activate-link", show_uri)
 
         frame = self.prefs
 
@@ -293,7 +369,7 @@ class PluginWindow(UniqueWindow, PersistentWindowMixin):
         model = ObjectStore()
         filter_model = ObjectModelFilter(child_model=model)
 
-        tv = PluginListView()
+        self._list_view = tv = PluginListView()
         tv.set_model(filter_model)
         tv.set_rules_hint(True)
 
@@ -301,26 +377,35 @@ class PluginWindow(UniqueWindow, PersistentWindowMixin):
 
         fb = Gtk.HBox(spacing=6)
 
-        filter_combo = PluginFilterCombo()
-        filter_combo.connect("changed", lambda s: filter_model.refilter())
-        fb.pack_start(filter_combo, False, True, 0)
+        enabled_combo = PluginEnabledFilterCombo()
+        enabled_combo.connect("changed", lambda s: filter_model.refilter())
+        enabled_combo.set_tooltip_text("Filter by plugin state / tag")
+        fb.pack_start(enabled_combo, True, True, 0)
+        self._enabled_combo = enabled_combo
 
-        filter_entry = ClearEntry()
+        type_combo = PluginTypeFilterCombo()
+        type_combo.connect("changed", lambda s: filter_model.refilter())
+        type_combo.set_tooltip_text("Filter by plugin type")
+        fb.pack_start(type_combo, True, True, 0)
+        self._type_combo = type_combo
+
+        filter_entry = UndoSearchEntry()
+        filter_entry.set_tooltip_text("Filter by plugin name or description")
         filter_entry.connect("changed", lambda s: filter_model.refilter())
-        filter_entry.enable_clear_button()
-        fb.pack_start(filter_entry, True, True, 0)
+        self._filter_entry = filter_entry
 
         sw.add(tv)
         sw.set_shadow_type(Gtk.ShadowType.IN)
-        sw.set_size_request(200, -1)
 
-        bbox = Gtk.HBox(homogeneous=True, spacing=12)
+        bbox = Gtk.VBox()
 
         errors = qltk.Button(_("Show _Errors"), Icons.DIALOG_WARNING)
         errors.set_focus_on_click(False)
         errors.connect('clicked', self.__show_errors)
+        errors.show()
+        errors = Align(errors, top=6, bottom=6)
         errors.set_no_show_all(True)
-        bbox.pack_start(Align(errors, border=6, right=-6), True, True, 0)
+        bbox.pack_start(errors, True, True, 0)
 
         pref_box = PluginPreferencesContainer()
 
@@ -328,12 +413,15 @@ class PluginWindow(UniqueWindow, PersistentWindowMixin):
             refresh = qltk.Button(_("_Refresh"), Icons.VIEW_REFRESH)
             refresh.set_focus_on_click(False)
             refresh.connect('clicked', self.__refresh, tv, pref_box, errors,
-                            filter_combo)
-            bbox.pack_start(Align(refresh, border=6), True, True, 0)
+                            enabled_combo)
+            bbox.pack_start(refresh, True, True, 0)
 
-        vbox.pack_start(Align(fb, border=6, right=-6), False, True, 0)
+        filter_box = Gtk.VBox(spacing=6)
+        filter_box.pack_start(fb, False, True, 0)
+        filter_box.pack_start(filter_entry, False, True, 0)
+        vbox.pack_start(Align(filter_box, border=6, right=-6), False, False, 0)
         vbox.pack_start(sw, True, True, 0)
-        vbox.pack_start(bbox, False, True, 0)
+        vbox.pack_start(Align(bbox, left=6), False, True, 0)
         paned.pack1(vbox, False, False)
 
         close = qltk.Button(_("_Close"), Icons.WINDOW_CLOSE)
@@ -355,15 +443,15 @@ class PluginWindow(UniqueWindow, PersistentWindowMixin):
             right_box.pack_start(bb_align, True, True, 0)
 
         paned.pack2(Align(right_box, border=12), True, False)
-        paned.set_position(250)
+        paned.set_position(275)
 
         self.add(paned)
 
-        self.__refill(tv, pref_box, errors, filter_combo)
+        self.__refill(tv, pref_box, errors, enabled_combo)
 
         self.connect('destroy', self.__destroy)
         filter_model.set_visible_func(
-            self.__filter, (filter_entry, filter_combo))
+            self.__filter, (filter_entry, enabled_combo, type_combo))
 
         self.get_child().show_all()
         filter_entry.grab_focus()
@@ -372,29 +460,31 @@ class PluginWindow(UniqueWindow, PersistentWindowMixin):
         tv.select_by_plugin_id(restore_id)
 
     def __filter(self, model, iter_, data):
+        """Filter a single row"""
         plugin = model.get_value(iter_)
         if not plugin:
             return False
 
-        entry, combo = data
+        entry, state_combo, type_combo = data
 
-        tag = combo.get_active_tag()
-        if tag:
+        plugin_type = type_combo.get_active_type()
+        if not issubclass(plugin.cls, plugin_type):
+            return False
+
+        tag_row = state_combo.get_active_row()
+        if tag_row:
             plugin_tags = plugin.tags
-            tag, flag = tag
-            pm = PluginManager.instance
-            enabled = pm.enabled(plugin) or not plugin.can_enable
-            if flag == ComboType.NO and plugin_tags or \
-                flag == ComboType.TAG and not tag in plugin_tags or \
-                flag == ComboType.EN and not enabled or \
-                flag == ComboType.DIS and enabled:
+            tag, flag = tag_row
+            enabled = plugin_enabled(plugin)
+            if (flag == EnabledType.NO and plugin_tags or
+                    flag == EnabledType.TAG and tag not in plugin_tags or
+                    flag == EnabledType.EN and not enabled or
+                    flag == EnabledType.DIS and enabled):
                 return False
 
         filter_ = entry.get_text().lower()
-        if not filter_ or filter_ in plugin.name.lower() or \
-                filter_ in (plugin.description or "").lower():
-            return True
-        return False
+        return (not filter_ or filter_ in plugin.name.lower()
+                or filter_ in (plugin.description or "").lower())
 
     def __destroy(self, *args):
         config.save()
@@ -409,6 +499,23 @@ class PluginWindow(UniqueWindow, PersistentWindowMixin):
         config.set("memory", "plugin_selection", plugin.id)
         container.set_plugin(plugin)
 
+    def unfilter(self):
+        """Clears all filters applied to the list"""
+
+        self._enabled_combo.set_active(0)
+        self._type_combo.set_active(0)
+        self._filter_entry.set_text(u"")
+
+    def move_to(self, plugin_id):
+        def selector(r):
+            return r[0].id == plugin_id
+
+        if self._list_view.select_by_func(selector):
+            return True
+        else:
+            self.unfilter()
+            return self._list_view.select_by_func(selector)
+
     def __plugin_toggled(self, tv, model, iter_, enabled):
         plugin = model.get_value(iter_)
         pm = PluginManager.instance
@@ -419,13 +526,13 @@ class PluginWindow(UniqueWindow, PersistentWindowMixin):
         riter = model.convert_iter_to_child_iter(iter_)
         rmodel.row_changed(rmodel.get_path(riter), riter)
 
-    def __refill(self, view, prefs, errors, combo):
+    def __refill(self, view, prefs, errors, state_combo):
         pm = PluginManager.instance
 
         # refill plugin list
         view.refill(pm.plugins)
 
-        # get all tags and refill combobox
+        # get all tags and refill tag-based (state) combobox
         tags = set()
         no_tags = False
         for plugin in pm.plugins:
@@ -433,18 +540,17 @@ class PluginWindow(UniqueWindow, PersistentWindowMixin):
                 no_tags = True
             tags.update(plugin.tags)
 
-        combo.refill(tags, no_tags)
+        state_combo.refill(tags, no_tags)
 
         if not len(pm.plugins):
             prefs.set_no_plugins()
 
         errors.set_visible(bool(pm.failures))
 
-    def __refresh(self, activator, view, prefs, errors, combo):
+    def __refresh(self, activator, view, prefs, errors, state_combo):
         pm = PluginManager.instance
         pm.rescan()
-
-        self.__refill(view, prefs, errors, combo)
+        self.__refill(view, prefs, errors, state_combo)
 
     def __show_errors(self, activator):
         pm = PluginManager.instance
